//=========================================================================//
//    TPrintDBGrid - freeware component                                    //
//    Preview and print DBGrid, RxDBGrid and other.                        //
//        DBGrid, RxDBGrid            //
//                                                                         //
//    :                                              //
//    E-Mail: zhabasov@mail.ru                                             //
//=========================================================================//

unit PrnDBGrid;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, DBGrids, ExtCtrls, Db, DBTables, Qrctrls, QuickRpt, QRPrnTr,
  printers, QRTee, Chart;

const
  MaxFieldsResult = 10;

type
  TPrintSetup = (psPrint, psPreview);

  TPrintDBGrid = class(TComponent)
  private
    FQRep: TQuickRep;
    FGrid: TDBGrid;
    FChart: TChart;
    FRepChart: TQRChart;
    FPageHeaderBand: TQRBand;
    FDetailBand: TQRBand;
    FGroupHeaderBand: TQRGroup;
    FGroupFooterBand: TQRBand;
    FSummaryBand: TQRBand;
    FHeaderFont: TFont;
    FDetailFont: TFont;
    FTitleFont: TFont;
    FTitle: string;
    FHeaderHeight: integer;
    FDetailHeight: integer;
    FTitleDate: boolean;
    FTitleNumPage: boolean;
    FTitleLeft: integer;
    FSetup: TPrintSetup;
    FResult: boolean;
    FSumBand: boolean;
    FResultOnly: boolean;

    RowCounts: array [0..MaxFieldsResult] of integer;
    ArrayResultFields: array [0..MaxFieldsResult] of integer;

    procedure SetTitleFont(Value: TFont);
    procedure SetHeaderFont(Value: TFont);
    procedure SetDetailFont(Value: TFont);
    procedure SetSetup(Value: TPrintSetup);
    function  GetSetup: TPrintSetup;
  protected
    REG_Decimal: char;
    CurNo: integer;
    CurSumm: currency;
    CurLeft: integer;
    procedure QRepPrint(Sender: TObject; var Value: String);
    procedure QRepBeforePrint(Sender: TQRCustomBand; var PrintBand: Boolean);
    procedure QRepSummBandBeforePrint(Sender: TQRCustomBand; var PrintBand: Boolean);
    procedure QRepAfterPrint(Sender: TQRCustomBand; BandPrinted: Boolean);
    procedure QReportBeforePrint(Sender: TCustomQuickRep; var PrintReport: Boolean);
  public
    SetOfResultFields: set of byte;
  published
    property DBGrid:TDBGrid read FGrid write FGrid;
    //property Chart:TChart read FChart write FChart;
    property HeaderFont:TFont read FHeaderFont write SetHeaderFont;
    property DetailFont:TFont read FDetailFont write SetDetailFont;
    property TitleFont:TFont read FTitleFont write SetTitleFont;
    property Title: string read FTitle write FTitle;
    property TitleDate: boolean read FTitleDate write FTitleDate default false;
    property TitleNumPage: boolean read FTitleNumPage write FTitleNumPage default false;
    property TitleLeft: integer read FTitleLeft write FTitleLeft;
    property HeaderHeight: integer read FHeaderHeight write FHeaderHeight;
    property DetailHeight: integer read FDetailHeight write FDetailHeight;
    property Setup: TPrintSetup read GetSetup write SetSetup;
    property Result: boolean read FResult write FResult default False;
    property SumBand: boolean read FSumBand write FSumBand default False;
    property ResultOnly: boolean read FResultOnly write FResultOnly default False;

    procedure Execute;
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  end;

  procedure Register;

implementation

uses registry;

procedure Register;
begin
  RegisterComponents('Data Controls', [TPrintDBGrid]);
end;

constructor TPrintDBGrid.Create;
var Reg: TRegIniFile;
    S: string;
begin
  inherited create(AOwner);
  FHeaderFont:= TFont.Create;
  FDetailFont:= TFont.Create;
  FTitleFont:= TFont.Create;
  FHeaderFont.Size:= 8;
  FDetailFont.Size:= 8;
  FHeaderHeight:= 50;
  FDetailHeight:= 20;
  FTitleLeft:= 50;
  Reg:=TRegIniFile.Create('Control Panel');
  S:= Reg.ReadString('International','sDecimal',',');
  REG_Decimal:= S[1];
  Reg.Free;
  SetOfResultFields:= [];
end;

destructor TPrintDBGrid.Destroy;
begin
  FHeaderFont.Free;
  FDetailFont.Free;
  FTitleFont.Free;
  inherited;
end;

procedure TPrintDBGrid.Execute;
var i, j, LeftPoint, k: integer;
begin
  CurNo:= -1;
  for i:=0 to MaxFieldsResult do RowCounts[i]:= 0;
  FQRep:= TQuickRep.Create(self);
  if FChart<>nil then
    begin
      FRepChart:= TQRChart.Create(FQRep);
      FRepChart.Chart.Assign(FChart);
    end;
  with FQRep do
    begin
      BeforePrint:= QReportBeforePrint;
      DataSet:= FGrid.DataSource.DataSet;
      FPageHeaderBand:= TQRBand.Create(FQRep);
      with FPageHeaderBand do
        begin
          BandType:= rbPageHeader;
          Parent:= FQRep;
          Height:= FHeaderHeight;
        end;

      FDetailBand:= TQRBand.Create(FQRep);
      with FDetailBand do
        begin
          BandType:= rbDetail;
          Parent:= FQRep;
          Height:= FDetailHeight;
          AfterPrint:= QRepAfterPrint;
          Enabled:= not FResultOnly; 
        end;

      if FResult then
      begin
        j:= 0;   //     
        for i:=0 to MaxFieldsResult do
          if i in SetOfResultFields then
            begin
              RowCounts[i]:= 0;
              ArrayResultFields[j]:= i;
              inc(j);
            end;

          for i:= 0 to j-1 do
            begin
              FGroupFooterBand:= TQRBand.Create(FQRep);
              with FGroupFooterBand do
                begin
                  BandType:= rbGroupFooter;
                  Parent:= FQRep;
                  Height:= FDetailHeight;
                  BeforePrint:= QRepBeforePrint;
                  Tag:= i;
                  if FResultOnly then Color:= clWindow
                  else Color:= $00EFEFEF;
                end;
              with TQRLabel.Create(FGroupFooterBand) do
                begin
                  Parent:= FGroupFooterBand;
                  Top:= 2;
                  Font:= FDetailFont;
                  Font.Style:= Font.Style+[fsBold];
                  Transparent:= true;
                  if FResultOnly then Left:= 300
                  else
                    begin
                      LeftPoint:= 0;
                      for k:= 0 to ArrayResultFields[i]-1 do LeftPoint:= LeftPoint+FGrid.Columns.Items[k].Width+1;
                      Left:= LeftPoint+10;
                    end;
                end;
              if FResultOnly then
              begin
              with TQRLabel.Create(FGroupFooterBand) do
                begin
                  Parent:= FGroupFooterBand;
                  Top:= 2;
                  Font:= FDetailFont;
                  Font.Style:= Font.Style;
                  Transparent:= true;
                  Left:= i*10+5;
                  Caption:= FGrid.Columns.Items[ArrayResultFields[i]].Title.Caption+' =';
                end;
              with TQRLabel.Create(FGroupFooterBand) do
                begin
                  Parent:= FGroupFooterBand;
                  Top:= 2;
                  Font:= FDetailFont;
                  Font.Style:= Font.Style;
                  Transparent:= true;
                  Left:= TQRLabel(TQRBand(Parent).Controls[1]).Width+i*10+10;
                end;
              end; //if FResultOnly then
              FGroupHeaderBand:= TQRGroup.Create(FQRep);
              with FGroupHeaderBand do
                begin
                  Parent:= FQRep;
                  Height:= 0;
                  Expression:= FGrid.DataSource.DataSet.Name+'.'+
                    FGrid.Columns.Items[ArrayResultFields[i]].FieldName;
                  FooterBand:= FGroupFooterBand;
                end;
            end;
      end;
      // 
      if FSumBand then
       begin
       FSummaryBand:= TQRBand.Create(FQRep);
       with FSummaryBand do
        begin
          BeforePrint:= QRepSummBandBeforePrint;
          BandType:= rbSummary;
          Parent:= FQRep;
          Height:= FDetailHeight;
          Color:= $00EFEFEF;
          with TQRLabel.Create(FSummaryBand) do
            begin
              Parent:= FSummaryBand;
              Left:= 10;
              Top:= 2;
              Font:= FDetailFont;
              Font.Style:= Font.Style+[fsBold];
              Caption:= ' :';
              Transparent:= true;
            end;
          with TQRSysData.Create(FSummaryBand) do
            begin
              Parent:= FSummaryBand;
              Left:= 100;
              Top:= 2;
              Font:= FDetailFont;
              Font.Style:= Font.Style+[fsBold];
              Data:= qrsDetailCount;
              Transparent:= true;
            end;
          with TQRLabel.Create(FSummaryBand) do
            begin
              Parent:= FSummaryBand;
              Left:= 100;
              Top:= 2;
              Font:= FDetailFont;
              Font.Style:= Font.Style+[fsBold];
              Caption:= '';
              Transparent:= true;
            end;
        end;
        end;
    end;

  LeftPoint:= 0;
  with FQRep do
    begin
      for i:=0 to FGrid.Columns.Count-1 do
        begin
          //PageHeader
          if not FResultOnly then
          with TQRLabel.Create(FPageHeaderBand) do
            begin
              AutoSize:= false;
              Caption:= FGrid.Columns.Items[i].Title.Caption;
              Frame.DrawBottom:= true;
              Frame.DrawTop:= true;
              Frame.DrawLeft:= true;
              Frame.DrawRight:= i=FGrid.Columns.Count-1;
              Width:= FGrid.Columns.Items[i].Width;
              Left:= LeftPoint;
              Font:= FHeaderFont;
              Top:= FHeaderHeight-Font.Size*2;
              Parent:= FPageHeaderBand;
              Alignment:= taCenter;
              Color:= $00E3E6D0;
            end;
          //Detail
          with TQRDBText.Create(FDetailBand) do
            begin
              if FGrid.Columns.Items[i].Field.DataType=ftCurrency then
                begin
                  OnPrint:= QRepPrint;
                  CurNo:= i;
                  CurLeft:= LeftPoint+5;
                  TQRLabel(FSummaryBand.Controls[2]).OnPrint:= QRepPrint;
                end;
              DataSet:= FGrid.DataSource.DataSet;
              DataField:= FGrid.Columns.Items[i].FieldName;
              AutoSize:= false;
              Frame.DrawTop:= true;
              Width:= FGrid.Columns.Items[i].Width;
              case FGrid.Columns.Items[i].Alignment of
                taRightJustify: Left:= LeftPoint-5;
                taLeftJustify:  Left:= LeftPoint+5;
                taCenter: Left:= LeftPoint;
              end;
              LeftPoint:= LeftPoint+FGrid.Columns.Items[i].Width+1;
              Top:= 0;
              Parent:= FDetailBand;
              Alignment:= FGrid.Columns.Items[i].Alignment;
              Font:= FDetailFont;
              Height:= FDetailHeight;
            end;
        end;
      //TitleCaption
      with TQRLabel.Create(FPageHeaderBand) do
        begin
          Parent:= FPageHeaderBand;
          Font:= FTitleFont;
          Left:= FTitleLeft;
          Top:= 10;
          Caption:= FTitle;
        end;
      //PageNum
      if FTitleNumPage then
      with TQRSysData.Create(FPageHeaderBand) do
        begin
          Parent:= FPageHeaderBand;
          Font.Size:= 8;
          Left:= 5;
          Top:= 15;
          Data:= qrsPageNumber;
        end;
      //Date
      if FTitleDate then
      with TQRSysData.Create(FPageHeaderBand) do
        begin
          Parent:= FPageHeaderBand;
          Font.Size:= 8;
          Left:= 0;
          Top:= 0;
          Data:= qrsDate;
        end;
      if LeftPoint < 780 then Page.Orientation:= poPortrait
      else Page.Orientation:= poLandscape;
      FGrid.DataSource.DataSet.DisableControls;
      FGrid.DataSource.DataSet.First;
      if FSetup=psPrint then Print
      else Preview;
      FGrid.DataSource.DataSet.First;
      FGrid.DataSource.DataSet.EnableControls;
    end;
  FPageHeaderBand.Free;
  FDetailBand.Free;
  FRepChart.Free;
  FQRep.Free;
end;

procedure TPrintDBGrid.QRepPrint(Sender: TObject; var Value: String);
begin
  if Pos(REG_Decimal,Value)>0 then Delete(Value,Pos(REG_Decimal,Value)+3,3);
  if (Pos(#32,Value)>0) or (Pos(#160,Value)>0) then exit;
  if Value='' then Value:= '0'+REG_Decimal+'00'
  else if (Pos(REG_Decimal,Value)=0) then Value:= Value+REG_Decimal+'00'
       else if Length(Copy(Value,Pos(REG_Decimal,Value)+1,2))<2 then Value:= Value+'0'
            else Value:= Copy(Value,1,Pos(REG_Decimal,Value)+2);
  Insert(' ',Value,Pos(REG_Decimal,Value)-3);
  Insert(' ',Value,Pos(REG_Decimal,Value)-7);
end;

procedure TPrintDBGrid.SetTitleFont(Value: TFont);
begin
  FTitleFont.Assign(Value);
end;

procedure TPrintDBGrid.SetHeaderFont(Value: TFont);
begin
  FHeaderFont.Assign(Value);
end;

procedure TPrintDBGrid.SetDetailFont(Value: TFont);
begin
  FDetailFont.Assign(Value);
end;

procedure TPrintDBGrid.SetSetup(Value: TPrintSetup);
begin
  FSetup:= Value;
end;

function TPrintDBGrid.GetSetup: TPrintSetup;
begin
  result:= FSetup
end;

procedure TPrintDBGrid.QRepBeforePrint(Sender: TQRCustomBand; var PrintBand: Boolean);
begin
  TQRLabel(Sender.Controls[0]).Caption:= IntToStr(RowCounts[Sender.Tag]);
  if FResultOnly then TQRLabel(Sender.Controls[2]).Caption:=
    FGrid.DataSource.DataSet.Fields[ArrayResultFields[Sender.Tag]].AsString;
  RowCounts[Sender.Tag]:= 0;
end;

procedure TPrintDBGrid.QRepAfterPrint(Sender: TQRCustomBand; BandPrinted: Boolean);
var i: integer;
begin
  if FResult then for i:=0 to MaxFieldsResult do RowCounts[i]:= RowCounts[i]+1;
  if CurNo>=0 then CurSumm:= CurSumm+FGrid.Columns.Items[CurNo].Field.AsCurrency;
end;

procedure TPrintDBGrid.QReportBeforePrint(Sender: TCustomQuickRep; var PrintReport: Boolean);
begin
  CurSumm:= 0;
end;

procedure TPrintDBGrid.QRepSummBandBeforePrint(Sender: TQRCustomBand; var PrintBand: Boolean);
begin
  if CurNo>=0 then
    begin
      TQRLabel(Sender.Controls[2]).Caption:= CurrToStr(CurSumm);
      TQRLabel(Sender.Controls[2]).Left:= CurLeft;
    end;  
end;

end.
